<?php

namespace Drupal\entity_reference_hierarchy\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_reference_hierarchy\EntityReferenceHierarchyFieldItemList;
use Drupal\entity_reference_hierarchy_revisions\EntityReferenceHierarchyRevisionsFieldItemList;

/**
 * Class EntityReferenceHierarchyFormatterTrait
 */
trait EntityReferenceHierarchyFormatterTrait {

  private function listTypes() {
    return [
      'ol' => $this->t('Ordered List'),
      'ul' => $this->t('Unordered List'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'list_type' => 'ol',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['list_type'] = [
      '#title' => t('Link label to the referenced entity'),
      '#type' => 'radios',
      '#default_value' => $this->getSetting('list_type'),
      '#options' => $this->listTypes(),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $list_types = $this->listTypes();
    $list_type = $this->getSetting('list_type');
    $summary[] = isset($list_types[$list_type]) ? $list_types[$list_type] : $list_types['ol'];

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    /** @var EntityReferenceHierarchyFieldItemList|EntityReferenceHierarchyRevisionsFieldItemList $items */

    $elements = parent::viewElements($items, $langcode);

    // Get outline.
    $outline = $items->getFieldHierarchyOutline();

    // Build list items according to the outline.
    $list_items = $this->getNestedListItemElements($items, $elements, $outline);

    return $this->viewNestedElements($list_items);
  }

  /**
   * Prepares items to be rendered by "item_list" theme.
   *
   * @param FieldItemListInterface $items
   * @param array $elements
   * @param array $delta_outline
   *
   * @return array
   */
  public function getNestedListItemElements(FieldItemListInterface $items, array $elements, array $delta_outline) {
    $result = [];

    foreach ($delta_outline as $delta => $branch) {
      $item = [$elements[$delta]];

      // Add children if available.
      if (!empty($branch['children'])) {
        $item[]['#items'] = $this->getNestedListItemElements($items, $elements, $branch['children']);
      }

      $result[] = $item;
    }

    return $result;
  }

  /**
   * Returns nested form elements.
   *
   * @param array $item_list_items
   *
   * @return array@return array
   */
  public function viewNestedElements(array $item_list_items) {
    return [
      '#theme' => 'item_list',
      '#list_type' => $this->getSetting('list_type'),
      '#items' => $item_list_items,
    ];
  }

}
